<?php

namespace Cursed\Input;

class InputParser
{
    private string $oldStty;

    private string $buffer = '';

    public bool $debug = false;

    public function __construct()
    {
        $this->oldStty = exec("stty -g");
        exec("stty raw -echo");
    }

    public function __destruct()
    {
        exec("stty {$this->oldStty}");
    }

    public function read(): ?InputEvent
    {
        $r = [ STDIN ]; $w = $e = [];
        if (stream_select($r, $w, $e, 0)) {
            $this->buffer .= fread(STDIN, 1024);
        }
        if (mb_strlen($this->buffer) == 0) {
            return null;
        }

        if (preg_match('/^(\eO[0-9]*?[~a-zA-Z])/', $this->buffer, $m)) {
            $l = mb_strlen($m[1]);
            $tok = mb_substr($this->buffer, 0, $l);
            $this->buffer = mb_substr($this->buffer, $l);
            $match = match ($tok) {
                "\eOP"  => new Key(Key::K_F1, false),
                "\eOQ"  => new Key(Key::K_F2, false),
                "\eOR"  => new Key(Key::K_F3, false),
                "\eOS"  => new Key(Key::K_F4, false),
                default => null,
            };
            if (!$match && $this->debug) {
                fwrite(STDERR, addcslashes($tok, "\e")."\r\n");
            }
            return $match;
        
        } elseif (preg_match('/^(\e\[[0-9;]*?[~a-zA-Z])/', $this->buffer, $m)) {
            $l = mb_strlen($m[1]);
            $tok = mb_substr($this->buffer, 0, $l);
            $this->buffer = mb_substr($this->buffer, $l);
            $match = match ($tok) {
                "\e[A"    => new Key(Key::K_UP,     false),
                "\e[B"    => new Key(Key::K_DOWN,   false),
                "\e[C"    => new Key(Key::K_RIGHT,  false),
                "\e[D"    => new Key(Key::K_LEFT,   false),
                "\e[2~"   => new Key(Key::K_INS,    false),
                "\e[3~"   => new Key(Key::K_DEL,    false),
                "\e[5~"   => new Key(Key::K_PGUP,   false),
                "\e[6~"   => new Key(Key::K_PGDN,   false),
                "\e[H"    => new Key(Key::K_HOME,   false),
                "\e[F"    => new Key(Key::K_END,    false),
                "\e[15~"  => new Key(Key::K_F5,     false),
                "\e[17~"  => new Key(Key::K_F6,     false),
                "\e[18~"  => new Key(Key::K_F7,     false),
                "\e[19~"  => new Key(Key::K_F8,     false),
                "\e[20~"  => new Key(Key::K_F9,     false),
                "\e[21~"  => new Key(Key::K_F10,    false),
                default   => null,
            };
            if (!$match && $this->debug) {
                fwrite(STDERR, addcslashes($tok, "\e")."\r\n");
            }
            return $match;
        } elseif (preg_match('/^(\e[a-zA-Z])/', $this->buffer, $m)) {
            $l = mb_strlen($m[1]);
            $tok = mb_substr($this->buffer, 0, $l);
            $this->buffer = mb_substr($this->buffer, $l);
            return new Key("[m-".$tok[1]."]", false);
        } else {
            $key = mb_substr($this->buffer, 0, 1);
            $this->buffer = mb_substr($this->buffer, 1);
            $ord = mb_ord($key);
            if ($ord >= 32 && $ord < 127) {
                return new Key($key, true);
            } else {
                return match ($ord) {
                    1 => new Key(Key::C_A, false),
                    2 => new Key(Key::C_B, false),
                    3 => new Key(Key::C_C, false),
                    4 => new Key(Key::C_D, false),
                    5 => new Key(Key::C_E, false),
                    6 => new Key(Key::C_F, false),
                    7 => new Key(Key::C_G, false),
                    8 => new Key(Key::C_H, false),
                    9 => new Key(Key::C_I, false),
                    10 => new Key(Key::C_J, false),
                    11 => new Key(Key::C_K, false),
                    12 => new Key(Key::C_L, false),
                    13 => new Key(Key::C_M, false),
                    14 => new Key(Key::C_N, false),
                    15 => new Key(Key::C_O, false),
                    16 => new Key(Key::C_P, false),
                    17 => new Key(Key::C_Q, false),
                    18 => new Key(Key::C_R, false),
                    19 => new Key(Key::C_S, false),
                    20 => new Key(Key::C_T, false),
                    21 => new Key(Key::C_U, false),
                    22 => new Key(Key::C_V, false),
                    23 => new Key(Key::C_W, false),
                    24 => new Key(Key::C_X, false),
                    25 => new Key(Key::C_Y, false),
                    26 => new Key(Key::C_Z, false),
                    27 => new Key(Key::K_ESC, false),
                    127 => new Key(Key::K_BS, false),
                    default => null
                };
            }
        }

    }

}

